# -*- coding: utf-8 -*-
# ikcpedit.py Ver.25
# N guage control panel editor by ik-gadgets
# コマンド等については下記def print_help(): 以下を参照

import sys
import pygame
import os
from pygame.locals import *
import re

pygame.init()


file_name = ""
string1 = ""
string2 = ""
window_w = 1080
window_h = 864
grid_unit = 13

sysfont = pygame.font.SysFont("yumincho", 16)
Nofont = pygame.font.SysFont("msgothic", 26)
controlfont = pygame.font.SysFont("msgothic", 40)
optionfont = pygame.font.SysFont("msgothic", 13,bold=True)

FPSCLOCK = pygame.time.Clock()
FPSCLOCK.tick(30) # flame rate
            
colorlist = [(0,0,0),(128,0,0),(255,0,0),(255,165,0),(255,255,0),
             (0,128,0),(0,0,255),(128,0,128),(128,128,128),(255,255,255)]
             # 黒、茶、赤、橙、黄、緑、青、紫、灰, 白
facilities = [] # [No,type,pos,...],[...
text_list = []

key = K_m
mode = key
mode_key_list = (K_l,K_p,K_o,K_s,K_a,K_b,K_t,K_c,K_m)
# K_l = draw lines
# K_p = add point
# K_o = add slow point
# K_s = add sensor
# K_a = add 3-lamp traffic light
# K_b = add 6-lamp traffic light
# K_t = add tile
# K_c = add controller
# K_m = move

mode_string_list = (
    "L: Mode = Draw lines : Power section = ",
    "P: Mode = Add Point",
    "O: Mode = Add Slow point",
    "S: Mode = Add Sensor",
    "A: Mode = Add 3-lamp traffic light",
    "B: Mode = Add 6-lamp traffic light",
    "T: Mode = Add Tile",
    "C: Mode = Add Controller : Power section = ",
    "M: Mode = Move selected items by mouse drag or arrow keys"
)

lines = []

# commamd
# 0 delete
# 1 move left
# 2 move right
# 3 move up
# 4 move down
command_key_list = (
    K_w,
    K_DELETE,
    K_LEFT,
    K_RIGHT,
    K_UP,
    K_DOWN,
    K_PAGEDOWN,
    K_PAGEUP,
    K_f,
    K_r,
    K_ESCAPE,
    K_RETURN,
    K_h
)

select_rect = pygame.Rect(0,0,1,1) # <-- select_rect.top = 0 の場合、選択なし。

command_string_list =(
    "Write Data to ",
    "Delete",
    "Move left",
    "Move right",
    "Move up",
    "Move down",
    "Panel size down",
    "Panel size up",
    "Flip",
    "Rotate",
    "Clear Selection",
    "Clear Selection",
    "Print Help to com window"
)

number_list = []

def index2color(i):
    if 0 <= i < len(colorlist):
        return colorlist[i]
    else:
        B = i%1000
        if B > 255: B = 255
        G = int(i/1000)%1000
        if G > 255: G = 255
        R = int(i/1000000)
        if R > 255: R = 255
        return (R,G,B)

def next_No():
    global number_list
    i = 1
    while(1):
        if i in number_list:
            i +=1
        else:
            number_list.append(i)
            return i

def delete_No(i):
    global number_list
    if i in number_list:
        number_list.remove(i)

def add_No(i):
    global number_list
    if i > 0:
        number_list.append(i)

class Line:
    def __init__(self,x1,y1,x2,y2,color_index=0,select_flag=0):
        self.x1 = x1
        self.y1 = y1
        self.x2 = x2
        self.y2 = y2
        self.color_index = color_index
        self.color = index2color(color_index)
        self.select_flag = select_flag

class Text:
    def __init__(self,x,y,font_name,font_size,bold,color_index,text,select_flag=0):
        self.x = x
        self.y = y
        self.font_name = font_name
        self.font_size = font_size
        self.bold = bold
        self.color_index = color_index
        self.color = index2color(color_index)
        self.text = text
        self.select_flag = select_flag
        self.font = pygame.font.SysFont(font_name,font_size,bold)
        (self.w,self.h) = self.font.size(text)
        self.bmp = self.font.render(self.text,True,self.color)

    def collide(self):
        if select_rect.top:
            if select_rect.collidepoint(self.x,self.y):
                self.select_flag = 1 # select flag on

    def flip(self):
        None

    def rotate(self):
        None

    def draw(self): # draw rectangle if selected
        BACK_PANEL.blit(self.bmp,(self.x,self.y))
        if (self.select_flag):
            sx = self.x - 2
            sy = self.y - 2
            sw = self.w + 2
            sh = self.h + 2
            if self.color_index == 2 or self.color_index == 3: # color = red or orange
                pygame.draw.rect(BACK_PANEL,colorlist[0],(sx,sy,sw,sh),2) # select line = black
            else:
                pygame.draw.rect(BACK_PANEL,colorlist[2],(sx,sy,sw,sh),2) # select line = red

    def string(self):
        tempstr = self.__class__.__name__ \
            + " " + str(self.x) \
            + " " + str(self.y) \
            + " yumincho" \
            + " " + str(int(self.font_size)) \
            + " " + str(self.bold) \
            + " " + str(self.color_index) \
            + ' "'+ self.text \
            + '"\n'
        return tempstr

class Base:
    def __init__(self,number,x,y,w=26,h=26,color_index=0,flip_flag=0,rotate_angle=0,select_flag=0):
        self.number = number
        self.x = x
        self.y = y
        self.w = w
        self.h = h
        self.color_index = color_index
        self.color = index2color(color_index)
        self.flip_flag = flip_flag
        self.rotate_angle = rotate_angle
        self.select_flag = select_flag

    def collide(self):
        if select_rect.top:
            if select_rect.collidepoint(self.x,self.y):
                self.select_flag = 1 # select flag on
#            else:
#                self.select_flag = 0 # select flag off

    def flip(self):
        None

    def rotate(self):
        None

    def draw(self): # draw rectangle if selected
        if (self.select_flag):
            sx = self.x - 2
            sy = self.y - 2
            if self.rotate_angle == 0 or self.rotate_angle == 180:
                sw = self.w + 2
                sh = self.h + 2
            else:
                sh = self.w + 2
                sw = self.h + 2
            if self.color_index == 2 or self.color_index == 3: # color = red or orange
                pygame.draw.rect(BACK_PANEL,colorlist[0],(sx,sy,sw,sh),2) # select line = black
            else:
                pygame.draw.rect(BACK_PANEL,colorlist[2],(sx,sy,sw,sh),2) # select line = red
            
class Tile(Base):
    def __init__(self,number,x,y,flip_flag=0,rotate_angle=0,w=26,h=26,color_index=0,select_flag=0,k=0,image_file=None):
        super().__init__(number,x,y,w,h,color_index,flip_flag,rotate_angle)
        self.image_file = image_file
        if image_file:
            if os.path.exists(image_file):
                self.image = pygame.image.load(image_file)
                self.w = self.image.get_width()
                self.h = self.image.get_height()
                self.k = k
                if k > 0:
                    self.h = int(self.h * k / self.w)
                    self.w = k
                    self.image = pygame.transform.scale(self.image,(self.w,self.h))
            else:
                print("### ERROR ### ",image_file," not found")
                pygame.quit()
                sys.exit()

    def draw(self):
        if self.image_file:
            im = self.image
            if self.flip_flag:
                im = pygame.transform.flip(im,True,False)
            if self.rotate_angle:
                im = pygame.transform.rotate(im,self.rotate_angle)
            self.w = im.get_width()
            self.h = im.get_height()
            BACK_PANEL.blit(im,(self.x,self.y))
            if self.select_flag:
                pygame.draw.rect(BACK_PANEL,colorlist[2],(self.x,self.y,self.w,self.h),2) # select line = red
        else:
            pygame.draw.rect(BACK_PANEL,self.color,(self.x,self.y,self.w,self.h),0)
            super().draw() # draw rectangle if selected

        if self.select_flag:
            bmp = Nofont.render(str(self.number), True, colorlist[0])
            BACK_PANEL.blit(bmp, (self.x,self.y-24))

    def flip(self):
        if self.image_file:
            self.flip_flag ^= 1

    def rotate(self):
        if self.image_file:
            self.rotate_angle += 90
            if self.rotate_angle >= 360:
                self.rotate_angle = 0
        else: 
            (self.w,self.h) = (self.h,self.w)

    def string(self):
        tempstr = "{:4}".format(self.number) \
            + " " + self.__class__.__name__ \
            + " " + str(int(self.x)) \
            + " " + str(int(self.y))
        if self.image_file:
            tempstr += " " + str(int(self.flip_flag)) \
                + " " + str(int(self.rotate_angle)) \
                + " " + str(int(self.k)) \
                + " " + self.image_file \
                + "\n"
        else:
            tempstr += " " + str(int(self.w)) \
                + " " + str(int(self.h)) \
                + " " + str(int(self.color_index)) \
                + "\n"
        return tempstr

class Point(Base):
    def __init__(self,number,x,y,w=26*2,h=26*2,flip_flag=0,rotate_angle=0,select_flag=0):
        super().__init__(number,x,y,w,h)
        self.flip_flag = flip_flag
        self.rotate_angle = rotate_angle

    def draw(self):
        draw_point(str(self.number),self.x,self.y,self.w,self.h,self.flip_flag,self.rotate_angle,self.__class__.__name__)
        super().draw() # draw rectangle if selected
            
    def flip(self):
        if self.flip_flag >= 3:
            self.flip_flag = 0
        else:
            self.flip_flag += 1

    def rotate(self):
        self.rotate_angle += 90
        if self.rotate_angle >= 360:
            self.rotate_angle = 0

    def string(self):
        tempstr = "{:4}".format(self.number) \
            + " " + self.__class__.__name__ \
            + " " + str(int(self.x)) \
            + " " + str(int(self.y)) \
            + " " + str(int(self.flip_flag)) \
            + " " + str(int(self.rotate_angle)) \
            + "\n"
        return tempstr

class Point_slow(Point):
    def __init__(self,number,x,y,w=26*2,h=26*2,flip_flag=0,rotate_angle=0,select_flag=0):
        super().__init__(number,x,y,w,h,flip_flag,rotate_angle)

class Controller(Base):
    def __init__(self,number,x,y,w=26*3,h=26*2,color_index=0,\
                 flip_flag=0,drive=0,acc=20,offset=0,wob_p=0,wob_t=10,current=0,select_flag=0):
        super().__init__(number,x,y,w,h,color_index)
        self.flip_flag = flip_flag
        self.drive = drive
        self.acc = acc
        self.offset = offset
        self.wob_p = wob_p
        self.wob_t = int(wob_t/10)*10
        if self.wob_t > 40:
            self.wob_t = 40
        elif self.wob_t < 10:
            self.wob_t = 10
        self.current = current

    def draw(self):
        draw_controller(str(self.number),self.x,self.y,self.w,\
                        self.h,self.color,self.flip_flag,self.drive,\
                        self.acc,self.offset,self.wob_t,self.wob_p,self.current)

        if (self.select_flag == 1):
            super().draw() # draw rectangle if selected
        elif (self.select_flag == 2):
            sx = self.x - 5
            sy = self.y + self.h + 2
            sw = 58
            sh = 20
            pygame.draw.rect(BACK_PANEL,colorlist[2],(sx,sy,sw,sh),2) # select line = red
        elif (self.select_flag == 3):
            sx = self.x +51
            sy = self.y + self.h + 2
            sw = 80
            sh = 20
            pygame.draw.rect(BACK_PANEL,colorlist[2],(sx,sy,sw,sh),2) # select line = red

        elif (self.select_flag == 4):
            sx = self.x -5
            sy = self.y + self.h + 20
            sw = 70
            sh = 20
            pygame.draw.rect(BACK_PANEL,colorlist[2],(sx,sy,sw,sh),2) # select line = red
        elif (self.select_flag == 5):
            sx = self.x +63
            sy = self.y + self.h + 20
            sw = 60
            sh = 20
            pygame.draw.rect(BACK_PANEL,colorlist[2],(sx,sy,sw,sh),2) # select line = red

            
    def flip(self):
        if self.flip_flag >= 3:
            self.flip_flag = 0
        else:
            self.flip_flag += 1

    def string(self):
        tempstr = "{:4}".format(self.number) \
            + " " + self.__class__.__name__ \
            + " " + str(int(self.x)) \
            + " " + str(int(self.y)) \
            + " " + str(int(self.color_index)) \
            + " " + str(int(self.flip_flag)) \
            + " " + str(int(self.drive)) \
            + " " + str(int(self.acc)) \
            + " " + str(int(self.offset)) \
            + " " + str(int(self.wob_p)) \
            + " " + str(int(self.wob_t)) \
            + " " + str(int(self.current)) \
            + "\n"
        return tempstr

class Sensor(Base):
    def __init__(self,number,x,y,w=26,h=26,data = 0,select_flag=0):
        super().__init__(number,x,y,w,h)
        self.data = data

    def draw(self):
        draw_sensor(str(self.number),self.x,self.y,self.w,self.h,self.data)
        super().draw() # draw rectangle if selected

    def string(self):
        tempstr = "{:4}".format(self.number) \
            + " " + self.__class__.__name__ \
            + " " + str(int(self.x)) \
            + " " + str(int(self.y)) \
            + " " + str(int(self.data)) \
            + "\n"
        return tempstr

class Signal3(Base):
    def __init__(self,number,x,y,w=26,h=26*3,flip_flag = 0,data = 1,select_flag=0):
        super().__init__(number,x,y,w,h)
        self.flip_flag = flip_flag
        self.data = data

    def flip(self):
        self.flip_flag ^= 1

    def rotate(self):
        self.data += 1
        if self.data > 3:
            self.data = 0

    def draw(self):
        draw_signal3(str(self.number),self.x,self.y,self.flip_flag,self.w,self.h,self.data)
        super().draw() # draw rectangle if selected

    def string(self):
        tempstr = "{:4}".format(self.number) \
            + " " + self.__class__.__name__ \
            + " " + str(int(self.x)) \
            + " " + str(int(self.y)) \
            + " " + str(int(self.flip_flag)) \
            + " " + str(int(self.data)) \
            + "\n"
        return tempstr

class Signal6(Base):
    def __init__(self,number,x,y,w=26*2,h=26*3.5,flip_flag = 0,data = 5,select_flag=0):
        super().__init__(number,x,y,w,h,select_flag)
        self.flip_flag = flip_flag
        self.data = data

    def flip(self):
        self.flip_flag ^= 1

    def rotate(self):
        self.data += 1
        if self.data > 15:
            self.data = 0

    def draw(self):
        draw_signal6(str(self.number),self.x,self.y,self.flip_flag,self.w,self.h,self.data)
        super().draw() # draw rectangle if selected

    def string(self):
        tempstr = "{:4}".format(self.number) \
            + " " + self.__class__.__name__ \
            + " " + str(int(self.x)) \
            + " " + str(int(self.y)) \
            + " " + str(int(self.flip_flag)) \
            + " " + str(int(self.data)) \
            + "\n"
        return tempstr

def find_facility(xpos,ypos):
    findlist = []
    for i in range(len(facilities)):
        w = facilities[i].w
        h = facilities[i].h
        x = facilities[i].x
        y = facilities[i].y
        if x <= xpos <= x+w:
            if y <= ypos <= y+h:
                findlist.append(i)
    return findlist


def print_help():
    print("")
    print("# -*- coding: utf-8 -*-")
    print("")
    print("【ファイル入力】")
    print("   プログラム起動時に引数としてファイル名を指定するとそのファイルを読み込む。")
    print("")
    print("   ファイル名を指定しない場合はcontrol_panel_番号.txtを探し")
    print("    番号が100以下で一番大きいファイルを選んで読み込む。")
    print("")
    print("    該当ファイルが無い場合は空のパネルが表示される。")
    print("")
    print("【ファイル出力】")
    print("   Wまたはwキーで編集結果をファイル名 control_panel_番号.txt に出力する。")
    print("   番号は１から100まで順に使われていないファイル名を探して記録する。")
    print("")
    print("【各モード共通操作】")
    print("   Page Upキーでパネルサイズを面積2倍に拡大、Page Downキーで面積1/2に縮小する。")
    print("")
    print("   マウス中央ボタンをドラッグ（押したまま移動)すると")
    print("   表示された赤枠内の物が選択される。")
    print("   その際以前の選択は解除されないので選択を追加してゆける。")
    print("")
    print("   選択されたラインは太く表示され、その他の選択されたアイテムには")
    print("   赤枠が表示される。")
    print("")
    print("   EnterまたはEscキーですべての選択が解除される。")
    print("")
    print("   Deleteキーで選択されたものが消去される。")
    print("")
    print("   Hまたはhキーでコマンドウインドウ側にこのHelpを表示する。")
    print("")
    print("【ライン描画モード】")
    print("   Lまたはlキーでライン（線路）追加モードになり、続けてLまたはlを押すごとに")
    print("     次に追加されるラインの色が変わる。")
    print("")
    print("   マウスの左ボタンクリックでラインが追加され、")
    print("   同じ場所を２度左ボタンクリックするか右ボタンクリックで終端される。")
    print("")
    print("   さらに右ボタンをクリックすると新しいラインから順に削除される。")
    print("")
    print("【各要素追加モード】")
    print("   Pまたはpキーでポイント(黄土色)追加モードになる。")
    print("   Oまたはoキーでスローポイント(緑色)追加モードになる。")
    print("   Sまたはsキーでセンサ追加モードになる。")
    print("   Aまたはaキーで３灯信号機追加モードになる。")
    print("   Bまたはbキーで６灯信号機追加モードになる。")
    print("   Cまたはcキーでコントローラー(パワーパック)追加モードになり")
    print("     続けてCを押すごとに次に追加されるコントローラーの色が変わる。")
    print("   Tまたはtキーでタイル(四角形)追加モードになる。")
    print("     続けてTまたはtを押すごとに次に追加されるタイルの色が変わる。")
    print("")
    print("   マウスの左ボタンクリックで上記モードに対応した対象物を追加する。")
    print("   マウスの右ボタンクリックで追加した対象物を新しいものから順に削除する。")
    print("")
    print("【moveモード】")
    print("   Mまたはmキーでmoveモードになる。")
    print("")
    print("   アイテム上でマウス左ボタンをクリックした場合はそのアイテムが選択される。")
    print("   選択されたアイテム上で右ボタンをクリックすると選択解除できる。")
    print("")
    print("   ライン以外のアイテムを新規に選択した場合、それ以前のアイテムの選択")
    print("   は解除される。")
    print("")
    print("   斜めのラインはライン中央部のクリックで選択または選択解除ができる。")
    print("")
    print("   左ボタンのドラッグで選択されたものが移動する。")
    print("")
    print("   上下左右の矢印キーにより選択されたものが１グリッドづつ移動する。")
    print("")
    print("   Fまたはｆキーで選択されている信号機は左右反転する。")
    print("")
    print("   Fまたはｆキーで選択されているポイントの接続方向の表示が変わる。")
    print("   さらにfを２回押すごとに画像表示と実際のポイントの接続方向の対応が反転し")
    print("   ID番号の色（黒/白）が変わる。")
    print("")
    print("   Fまたはｆキーで選択されているコントローラは出力極性が変わる。")
    print("   さらに２回押すごとに画像表示と実際の出力極性の対応が反転し")
    print("   ID番号の色（黒/白）が変わる。")
    print("")
    print("   Ｒまたはｒキーで選択されたポイントが90°回転する。")
    print("   Ｒまたはｒキーで選択されたsignal3およびsignal6の信号色が変化する。")
    print("")
    print("   0～9キーで選択されたポイント、センサー、信号機、コントローラーの最大3桁の")
    print("   ID番号が下の桁からシフト入力される。")
    print("")
    print("   シリアルネットワークに接続されたハードウエアに対応するものは")
    print("   接続順に1から順に割り振る必要がある。")
    print("")
    print("【画像ファイルの追加方法】")
    print("   pngやjpgなどの画像ファイルの付加機能は本エディタに無いが")
    print("   入力ファイルに直接テキストエディタで追加すれば")
    print("   本エディタで表示、選択、移動、削除ができる。")
    print("")
    print("   [画像ファイル参照の記述例]")
    print("    3 tile 150 69 0 0 120 icon.png")
    print("")
    print("    各項目の意味は")
    print("    ID番号　tile　x座標　y座標　鏡像反転(0/1)　回転角(0/90/180/270)")
    print("                x方向サイズ　画像ファイル名(png,jpgなど)")
    print("    x方向サイズに0を指定すると画像ファイルのサイズを変更せずに表示する。")
    print("")
    print("【文字の追加方法】")
    print("   文字の付加機能は本エディタに無いが")
    print("   入力ファイルに直接テキストエディタで追加すれば")
    print("   本エディタで表示、選択、移動、削除ができる。")
    print("")
    print("   [文字の記述例]")
    print('    text 20 13 yumincho 26 1 0 "【Nゲージ制御盤】"')
    print("")
    print("    各項目の意味は")
    print("       text　x座標　y座標　フォント名　フォントサイズ　bold指定(0/1)")
    print('                      色指定　"文字列"')
    print("")
    print('    日本文字コードはutf-8。文字コード変換や別の文字コード対応は')
    print('    "python 文字コード" などでネット検索されたし。')
    print("")
    print("    使えるフォント名については以下のpythonプログラムで表示できる。")
    print("       import pygame")
    print("       for x in pygame.font.get_fonts():print(x)")
    print("")
    print("【入力ファイル内での色指定方法】")
    print("   line,text,controller,tileの色指定は")
    print("   黒=0、茶=1、赤=2、橙=3、黄=4、緑=5、青=6、紫=7、灰=8、白=9")
    print("   の数字１桁の他に")
    print("   RGBを各10進3桁0～255で表し連結して9桁にしたものでも良い。")
    print("")
    print("【ウインドウやグリッドサイズの詳細変更方法】")
    print("   入力ファイル内のWindow_w, window_h, grid の各キーワードに続く数字を")
    print("   テキストエディタで修正すれば")
    print("   ウィンドウサイズやグリッドサイズをピクセル単位で変更できる。")
    print("")

def search_control_file():
    file_head = "control_panel_"
    for file_No in range(100,0,-1):
        file_name = file_head + str(file_No) + ".txt"
        if os.path.exists(file_name):
            return file_name
    return None

def get_write_file_name():
    file_head = "control_panel_"
    file_No = 0
    while(1):
        file_No += 1
        file_name = file_head + str(file_No) + ".txt"
        if not os.path.exists(file_name):
            return file_name        
        if file_No == 100:
            print("### WARNING ### overwrite ",file_name)
            return file_name

def writefile():
    global window_h
    global window_w
    global grid_unit
    global lines
    global file_name

    head_lines = []
    line_lines = []
    text_lines = []
    facility_str = []
    print()
    file_name = get_write_file_name()
    print("Output file = ",file_name)
    print()

    head_lines.append("window_w " + str(window_w) + "\n")
    head_lines.append("window_h " + str(window_h) + "\n")
    head_lines.append("grid " + str(grid_unit) + "\n")

    for L in lines:
        line_lines.append("line "+ str(int(L.x1)) +" "+ str(int(L.y1)) +" "+ str(int(L.x2)) +" "+ str(int(L.y2)) +" "+ str(L.color_index) + "\n")

    for T in text_list:
        text_lines.append(T.string())

    facility_lines = []
    tile_lines = []
    number_list = []
    number_max = 0
    for facility in facilities:
        i = facility.number
        errstr = ""
        tmpstr = ""
        if facility.__class__.__name__ != "Tile":
            if i <= 0:
                errstr = "### ERROR ### id number <= 0\n"
                print(errstr,end="")
            elif i in number_list:
                errstr = "### ERROR ### Duplicate id number = " + str(i) + "\n"
                print(errstr,end="")
            number_list.append(i)
            if i > number_max: number_max = i
            tempstr = facility.string()
            facility_lines.append(tempstr.lower() + errstr)
        else:
            tempstr = facility.string()
            tile_lines.append(tempstr.lower() + errstr)

    for i in range(1,number_max+1):
        if i not in number_list:
            errstr = "### ERROR ### Missing id number = " + str(i) + "\n"
            print(errstr,end="")
            facility_lines.append(errstr)

    facility_lines.sort()

    f = open(file_name, 'w', encoding='UTF-8')
    f.writelines(head_lines)
    f.writelines(text_lines)
    f.writelines(facility_lines)
    f.writelines(line_lines)
    f.writelines(tile_lines)
    f.close()

def check_facility_number():
    number_max = 0
    number_list=[]
    for facility in facilities:
        i = facility.number
        if facility.__class__.__name__ != "Tile":
            if i <= 0:
                print("### ERROR ### ID number = 0")
            elif i in number_list:
                print("### ERROR ### Duplicate id number = " + str(i))
            number_list.append(i)
            if i > number_max: number_max = i
    for i in range(1,number_max+1):
        if i not in number_list:
            print("### ERROR ### Missing id number = " + str(i))

def read_line_error(line_No,errline):
    print("### ERROR ###: line ",line_No,":",errline)
    pygame.quit()
    print()
    sys.exit()
    
            
def readfile(file_list):
    global window_h
    global window_w
    global grid_unit
    global lines

    datalist = []
    print()
    for file in file_list:
      f = open(file,'r',encoding='UTF-8')
      datalist = datalist + f.readlines()
      f.close()
      print("input file = ",file)
    print()

    readline_cnt = 0
    lines = []
    for data in datalist:
        readline_cnt += 1
        print(str(readline_cnt),":",data,end="")
        tempdata = re.sub('#.*','',data)
        tempdata = tempdata.lower()
        splitdata = tempdata.split()
        if len(splitdata) == 0:
            continue
        elif len(splitdata) < 2:
            read_line_error(readline_cnt,data)
        elif splitdata[0] == "window_w":
            if len(splitdata) == 2:
                window_w = int(splitdata[1])
            else:
                read_line_error(readline_cnt,data)
        elif splitdata[0] == "window_h":
            if len(splitdata) == 2:
                window_h = int(splitdata[1])
            else:
                read_line_error(readline_cnt,data)
        elif splitdata[0] == "grid":
            if len(splitdata) == 2:
                grid_unit = int(splitdata[1])
            else:
                read_line_error(readline_cnt,data)
        elif splitdata[0] == "text":
            if len(splitdata) >= 8:
                xxx = re.search(r'"(.+)"',data)
                if xxx:
                    textstr = xxx.group(1)
                else:
                    textstr = splitdata[7]
                text_list.append(Text(
                        x=int(splitdata[1]),\
                        y=int(splitdata[2]),\
                        font_name=splitdata[3],\
                        font_size=int(splitdata[4]),\
                        bold=int(splitdata[5]),\
                        color_index=int(splitdata[6]),\
                        text=textstr))
            else:
                read_line_error(readline_cnt,data)
        elif splitdata[0] == "line":
            if len(splitdata) == 6:
                lines.append(
                    Line(\
                        int(splitdata[1]),\
                        int(splitdata[2]),\
                        int(splitdata[3]),\
                        int(splitdata[4]),\
                        int(splitdata[5])\
                    )
                )
            else:
                read_line_error(readline_cnt,data)
        elif splitdata[1] == "tile":
            if len(splitdata) == 7:
                facilities.append(Tile(int(splitdata[0]),int(splitdata[2]),int(splitdata[3]),w=int(splitdata[4]),h=int(splitdata[5]),color_index=int(splitdata[-1])))
            elif len(splitdata) == 5:
                facilities.append(Tile(int(splitdata[0]),int(splitdata[2]),int(splitdata[3]),\
                        image_file=splitdata[4]))
            elif len(splitdata) == 8:
                facilities.append(Tile(int(splitdata[0]),int(splitdata[2]),int(splitdata[3]),\
                        flip_flag=int(splitdata[4]),rotate_angle=int(splitdata[5]),k=int(splitdata[6]),\
                        image_file=splitdata[7]))
            else:
                read_line_error(readline_cnt,data)
        elif splitdata[1] == "point":
            add_No(int(splitdata[0]))
            if len(splitdata) == 6:
                angle = int(splitdata[5])
                angle /= 90
                angle = int(angle)*90
                angle %= 360
                facilities.append(Point(int(splitdata[0]),int(splitdata[2]),int(splitdata[3]),flip_flag=int(splitdata[4]),rotate_angle=angle))
            else:
                read_line_error(readline_cnt,data)
        elif splitdata[1] == "point_slow":
            add_No(int(splitdata[0]))
            if len(splitdata) == 6:
                angle = int(splitdata[5])
                angle /= 90
                angle = int(angle)*90
                angle %= 360
                facilities.append(Point_slow(int(splitdata[0]),int(splitdata[2]),int(splitdata[3]),flip_flag=int(splitdata[4]),rotate_angle=angle))
            else:
                read_line_error(readline_cnt,data)
        elif splitdata[1] == "controller":
            add_No(int(splitdata[0]))
            if len(splitdata) == 12:
                facilities.append(Controller(
                    number = int(splitdata[0]),\
                    x = int(splitdata[2]),\
                    y = int(splitdata[3]),\
                    color_index = int(splitdata[4]),\
                    flip_flag = int(splitdata[5]),\
                    drive = int(splitdata[6]),\
                    acc = int(splitdata[7]),\
                    offset = int(splitdata[8]),\
                    wob_p = int(splitdata[9]),\
                    wob_t = int(splitdata[10]),\
                    current = int(splitdata[11])\
                    )
                )
            else:
                read_line_error(readline_cnt,data)
        elif splitdata[1] == "sensor":
            add_No(int(splitdata[0]))
            if len(splitdata) == 5:
                facilities.append(Sensor(int(splitdata[0]),int(splitdata[2]),int(splitdata[3]),data=int(splitdata[4])))
            else:
                read_line_error(readline_cnt,data)
        elif splitdata[1] == "signal3":
            add_No(int(splitdata[0]))
            if len(splitdata) == 6:
                facilities.append(Signal3(int(splitdata[0]),int(splitdata[2]),int(splitdata[3]),\
                                          flip_flag=int(splitdata[4]),data=int(splitdata[5])))
            else:
                read_line_error(readline_cnt,data)
        elif splitdata[1] == "signal6":
            add_No(int(splitdata[0]))
            if len(splitdata) == 6:
                facilities.append(Signal6(int(splitdata[0]),int(splitdata[2]),int(splitdata[3]),\
                                          flip_flag=int(splitdata[4]),data=int(splitdata[5])))
            else:
                print("### ERROR ### line ",readline_cnt,":",datalist[readline_cnt-1])
                pygame.quit()
        else:
            read_line_error(readline_cnt,data)
    check_facility_number()
    print()
    for file in file_list:
      print("loaded input file = ",file)
    print()


def draw_back():
    global window_w
    global window_h
    global grid_unit

    # 背景色で塗りつぶし
    BACK_PANEL.fill((200, 200, 200))

    # 点を描画
    for ypos in range(0, window_h - 60, grid_unit):
        for xpos in range(0, window_w, grid_unit):
            pygame.draw.ellipse(BACK_PANEL, (64,64,64),(xpos,ypos,2,2))

def draw_firstpoint(firstpoint):
    if firstpoint:
        pygame.draw.circle(BACK_PANEL,(255,0,0),firstpoint,5)

def draw_controller(No_str,x,y,w,h,color,flip_flag,drive,acc,offset,wob_t,wob_p,current):
    drv_str = str(drive)
    if len(drv_str) < 3:
        for i in range(3-len(drv_str)):
            drv_str = " " + drv_str
    if flip_flag & 1:
        L_fill,R_fill = 0,2
    else:
        L_fill,R_fill = 2,0
    # 中央の四角
    pygame.draw.rect(BACK_PANEL,color,(x,y,w,h),3)
    # 左の三角
    pygame.draw.polygon(BACK_PANEL,color,((x,y),(x,y+h),(x-int(w/3),y+int(h/2))),L_fill)
    # 右の三角
    pygame.draw.polygon(BACK_PANEL,color,((x+w,y),(x+w,y+h),(x+w+int(w/3),y+int(h/2))),R_fill)
    bmp = Nofont.render(No_str,True,colorlist[0])
    if flip_flag & 2:
        bmp = Nofont.render(No_str, True, colorlist[9])
    else:
        bmp = Nofont.render(No_str, True, colorlist[0])
    BACK_PANEL.blit(bmp,(x,y-int(h/2)))

    if acc > 0:
        bmp = controlfont.render(drv_str, True, color)
        BACK_PANEL.blit(bmp,(x+int(w/8),y+int(h/8)))
    else:
        bmp = controlfont.render("N/A", True, colorlist[2])
        BACK_PANEL.blit(bmp,(x+int(w/3),y+int(h/8)))
        
    # Acc 0-126,0=Manual mode
    # Offset 0-126
    # Wob: 10,20,30,40ms/100,200,300,400ms, 0-62
    if acc == 0:
        bmp = optionfont.render("Acc:{:<3}".format(acc), True, colorlist[2])
    else:
        bmp = optionfont.render("Acc:{:<3}".format(acc), True, colorlist[0])
    BACK_PANEL.blit(bmp,(x,y+h+5))
    bmp = optionfont.render(" Offset:{:<2}".format(offset), True, colorlist[0])
    BACK_PANEL.blit(bmp,(x+50,y+h+5))
    bmp = optionfont.render("WobT:{:<2}ms WobP:{:<2}".format(wob_t,wob_p), True, colorlist[0])
    BACK_PANEL.blit(bmp,(x,y+h+23))
    # current
    bmp = optionfont.render("{:>4}mA".format(current), True, colorlist[0])
    BACK_PANEL.blit(bmp,(x+50,y-15))

def draw_point(No_str,x,y,w,h,flip,rotate,name):
    line_width = 6

    if name == "Point":
        fill_color = (189,183,107)
    else:
        fill_color = (107,183,189)

    if flip & 2:
        bmp = Nofont.render(No_str, True, colorlist[9])
    else:
        bmp = Nofont.render(No_str, True, colorlist[0])
    if flip & 1:
        if rotate == 0:
            pygame.draw.polygon(BACK_PANEL,fill_color,((x,y+h/2),(x+w,y),(x+w,y+h)))
            pygame.draw.line(BACK_PANEL,colorlist[0],(x,y+h/2),(x+w,y+h),width=line_width)
            BACK_PANEL.blit(bmp, (x+w/2+6,y+h/2-12))
        elif rotate == 90:
            pygame.draw.polygon(BACK_PANEL,fill_color,((x+h/2,y),(x+h,y+w),(x,y+w)))
            pygame.draw.line(BACK_PANEL,colorlist[0],(x+h/2,y),(x,y+w),width=line_width)
            BACK_PANEL.blit(bmp, (x+h/2-6,y+w/2-6))
        elif rotate == 180:
            pygame.draw.polygon(BACK_PANEL,fill_color,((x+w,y+h/2),(x,y+h),(x,y)))
            pygame.draw.line(BACK_PANEL,colorlist[0],(x+w,y+h/2),(x,y),width=line_width)
            BACK_PANEL.blit(bmp, (x+w/2-16,y+h/2-12))
        elif rotate == 270:
            pygame.draw.polygon(BACK_PANEL,fill_color,((x+h/2,y+w),(x,y),(x+h,y)))
            pygame.draw.line(BACK_PANEL,colorlist[0],(x+h/2,y+w),(x+h,y),width=line_width)
            BACK_PANEL.blit(bmp, (x+h/2-6,y+w/2-20))
    else:
        if rotate == 0:
            pygame.draw.polygon(BACK_PANEL,fill_color,((x,y+h/2),(x+w,y),(x+w,y+h)))
            pygame.draw.line(BACK_PANEL,colorlist[0],(x+w,y),(x,y+h/2),width=line_width)
            BACK_PANEL.blit(bmp, (x+w/2+6,y+h/2-12))
        elif rotate == 90:
            pygame.draw.polygon(BACK_PANEL,fill_color,((x+h/2,y),(x+h,y+w),(x,y+w)))
            pygame.draw.line(BACK_PANEL,colorlist[0],(x+h/2,y),(x+h,y+w),width=line_width)
            BACK_PANEL.blit(bmp, (x+h/2-6,y+w/2-6))
        elif rotate == 180:
            pygame.draw.polygon(BACK_PANEL,fill_color,((x+w,y+h/2),(x,y+h),(x,y)))
            pygame.draw.line(BACK_PANEL,colorlist[0],(x+w,y+h/2),(x,y+h),width=line_width)
            BACK_PANEL.blit(bmp, (x+w/2-16,y+h/2-12))
        elif rotate == 270:
            pygame.draw.polygon(BACK_PANEL,fill_color,((x+h/2,y+w),(x,y),(x+h,y)))
            pygame.draw.line(BACK_PANEL,colorlist[0],(x+h/2,y+w),(x,y),width=line_width)
            BACK_PANEL.blit(bmp, (x+h/2-6,y+w/2-20))

def draw_sensor(No_str,x,y,w,h,data):
    Green = (0,100,0)
    Magenta = (255,0,255)
    if data:
        pygame.draw.rect(BACK_PANEL,Magenta,(x,y,w,h),0)
    else:
        pygame.draw.rect(BACK_PANEL,Green,(x,y,w,h),0)
    bmp = Nofont.render('s', True, colorlist[9])
    BACK_PANEL.blit(bmp, (int(x+w/3.5),int(y-h/8)))
    bmp = Nofont.render(No_str, True, colorlist[0])
    BACK_PANEL.blit(bmp, (x,y-26))

def draw_signal3(No_str,x,y,flip,w,h,data):
    lamp_G = colorlist[0]
    lamp_Y = lamp_G
    lamp_R = lamp_G
    if data == 3:
        lamp_G = (0,255,255)
    elif data == 2:
        lamp_Y = (255,255,000)
    elif data == 1:
        lamp_R = (255,0,0)
    y += int(h/6)
    if flip:
        delta = 9
    else:
        delta = -9
    pygame.draw.rect(BACK_PANEL,(150,150,150),(x+delta,y,w,int(h*2/3)),0)
    pygame.draw.circle(BACK_PANEL,(150,150,150),(x+delta+w/2,y),w/2,0)
    pygame.draw.circle(BACK_PANEL,(150,150,150),(x+delta+w/2,y+h*2/3),w/2,0)

    pygame.draw.rect(BACK_PANEL,(60,60,60),(x,y,w,int(h*2/3)),0)
    pygame.draw.circle(BACK_PANEL,(60,60,60),(x+w/2,y),w/2,0)
    pygame.draw.circle(BACK_PANEL,(60,60,60),(x+w/2,y+h*2/3),w/2,0)

    pygame.draw.circle(BACK_PANEL,lamp_G,(x+w/2,y),w/2.2,0)
    pygame.draw.circle(BACK_PANEL,lamp_Y,(x+w/2,y+h/3),w/2.2,0)
    pygame.draw.circle(BACK_PANEL,lamp_R,(x+w/2,y+h*2/3),w/2.2,0)
    bmp = Nofont.render(No_str, True, colorlist[0])
    BACK_PANEL.blit(bmp, (x,y-39))

def draw_signal6(No_str,x,y,flip,w,h,data):
    dataL = data>>2
    dataR = data & 3
    if flip:
        offsetL = 0
        offsetR = h/7
        draw_signal3(No_str,x,y+offsetL,flip,w/2,h*6/7,dataL)
        draw_signal3("",x+w/2,y+offsetR,flip,w/2,h*6/7,dataR)
    else:
        offsetL = h/7
        offsetR = 0
        draw_signal3("",x+w/2,y+offsetR,flip,w/2,h*6/7,dataR)
        draw_signal3(No_str,x,y+offsetL,flip,w/2,h*6/7,dataL)

def delete_facilities():
    global facilities
    temp_list = []
    for facility in facilities:
        if not facility.select_flag:
            temp_list.append(facility)
        else:
            delete_No(facility.number)
    facilities = temp_list

def delete_lines():
    global lines
    temp_lines = []
    for L in lines:
        if not L.select_flag:
            temp_lines.append(L)
    lines = temp_lines

def move_lines(x,y):
    global lines
    for L in lines:
        if L.select_flag:
            (L.x1,L.y1) = (L.x1+x,L.y1+y)
            (L.x2,L.y2) = (L.x2+x,L.y2+y)

def move_text(x,y):
    global text_list
    for t in text_list:
        if t.select_flag:
            t.x += x
            t.y += y

def move_facilities(x,y):
    global facilities
    for facility in facilities:
        if facility.select_flag:
            facility.x += x
            facility.y += y

def update_facility_No(key):
    global facilities
    for facility in facilities:
        if facility.select_flag: # select flag = 1
            facility.number = facility.number*10%1000 + key - K_0

def write_bottom_column(key,colorindex):
    global window_w
    global window_h
    global string1
    global string2
    global mode
    global sysfont

    rect_line1 = pygame.draw.rect(BACK_PANEL, (255,255,255),(0, window_h-60, window_w, 60))
    if key == 0: # erase error string on mouse button down
        string2 = ""
    elif key in mode_key_list:
        string2 = ""
        string1 = mode_string_list[mode_key_list.index(key)]
        if mode == K_l or mode == K_c:
            string1 += str(colorindex)
        if mode == K_l or mode == K_c or mode == K_t:
            string1 += "  Color = "
            textsize = sysfont.size(string1)
            next_x_pos = textsize[0]+20
            rect_line2 = pygame.draw.rect(BACK_PANEL, index2color(colorindex),(next_x_pos, window_h-50, 24, 20))
            pygame.draw.rect(BACK_PANEL, (0,0,0),(next_x_pos, window_h-50, 24, 20),2)
    elif key in command_key_list:
        string2 = ""
        command = command_key_list.index(key)
        string2 = command_string_list[command]
        if key == K_w:
            string2 += file_name
    elif K_0 <= key <= K_9:
        string2 = ""
    else:
        try:
            string2 = "### Key ERROR ###  Input = " + str.upper(chr(key)) + "  (Available key = L,P,S,A,B,T,C,W,Delete,0-9,up,down,left,right,F,R,Esc,Enter,H)"
        except ValueError as error:
            string2 = ""
    bmp = sysfont.render(string1, True, (0, 0, 0))
    BACK_PANEL.blit(bmp, (20, window_h-50))
    bmp = sysfont.render(string2, True, (0, 0, 0))
    BACK_PANEL.blit(bmp, (20, window_h-25))

def pos2rect(pos1,pos2):
    x1 = min(pos1[0],pos2[0])
    y1 = min(pos1[1],pos2[1])
    x2 = max(pos1[0],pos2[0])
    y2 = max(pos1[1],pos2[1])
    return (x1,y1),(x2-x1,y2-y1)

def clear_select():
    global select_rect
    select_rect = pygame.Rect(0,0,1,1) # erase rectangle for selection
    firstpoint = []
    for L in lines:
        L.select_flag = 0 # select flag off
    for F in facilities:
        F.select_flag = 0
    for T in text_list:
        T.select_flag = 0

def main():

    global mode
    global BACK_PANEL
    global window_h
    global window_w
    global grid_unit
    global select_rect
    global string1
    global string2
    global icons
    global key
    global sysfont
    global Nofont
    global controlfont

    sel_pos = [0,0]

    args = sys.argv
    file_name_list = []
    print()
    if len(args) >= 2:
        for i in range(1,len(args)):
            file_name = args[i]
            if re.search(r'\.txt$',file_name):
                if os.path.exists(file_name):
                    file_name_list.append(file_name)                    
                else:
                    print("### ERROR ### file",args[i],"not found")
                    print()
                    pygame.quit()
                    sys.exit()
            else:
                print("### ERROR ###",args[i],"? argument must be *.txt")
                print()
                pygame.quit()
                sys.exit()
    if file_name_list:
        readfile(file_name_list)
    else:
        file_name = search_control_file()
        if file_name:
            readfile([file_name])
        else:
            print("Input file not found")
            print()
            text_list.append(Text(
                x=20,\
                y=13,\
                font_name="yumincho",\
                font_size=26,\
                bold=1,\
                color_index=0,\
                text="Control Panel"))

    BACK_PANEL = pygame.display.set_mode((window_w,window_h))
    pygame.display.set_caption("N guage control panel editor by ik-gadgets")

    colorindex = 0
    firstpoint = []
    draw_back()
    eventflag = 1
    write_bottom_column(mode,0)
    
    while True:
    # マウス/キーイベント処理
        for event in pygame.event.get():
            eventflag = 1
            if event.type == QUIT:
                pygame.quit()
                sys.exit()
            if event.type == MOUSEMOTION:
                xpos = round(event.pos[0]/grid_unit,0)*grid_unit
                ypos = round(event.pos[1]/grid_unit,0)*grid_unit

                pressed_button = pygame.mouse.get_pressed()
                if pressed_button[1] and sel_pos[0]:
                    select_rect = pygame.Rect(pos2rect(sel_pos,event.pos))
                    for facility in facilities:
                        facility.collide()
                    for t in text_list:
                        t.collide()
                    for L in lines:
                        if select_rect.top:
                            if select_rect.collidepoint((L.x1,L.y1)) and select_rect.collidepoint((L.x2,L.y2)):
                                L.select_flag = 1 # set select flag
                elif mode == K_m and pressed_button[0]: # move mode
                    delta_x = xpos - prev_x
                    delta_y = ypos - prev_y
                    move_lines(delta_x,delta_y)
                    prev_x = xpos
                    prev_y = ypos
                    delta_x = event.pos[0] - prev_nongrid_x
                    delta_y = event.pos[1] - prev_nongrid_y
                    move_facilities(delta_x,delta_y)
                    move_text(delta_x,delta_y)
                    prev_nongrid_x = event.pos[0]
                    prev_nongrid_y = event.pos[1]
            elif event.type == MOUSEBUTTONDOWN:
                key = 0 # to erase string2
                button = event.button
                xpos = round(event.pos[0]/grid_unit,0)*grid_unit
                ypos = round(event.pos[1]/grid_unit,0)*grid_unit
                prev_x = xpos # for move
                prev_y = ypos # for move
                prev_nongrid_x = event.pos[0] # for move
                prev_nongrid_y = event.pos[1] # for move
                string2 = ""
                if button == 2: # start drag select
                    sel_pos = event.pos
                elif button == 3:
                    select_rect = pygame.Rect(0,0,1,1) # erase rectangle for selection
                if mode == K_m:
                    prev_x = xpos
                    prev_y = ypos
                    for facility in facilities:
                        w = facility.w
                        h = facility.h
                        x = facility.x
                        y = facility.y
                        if (x <= xpos <= x+w) and (y <= ypos <= y+h):
                            if button == 1:
                                if facility.select_flag == 0:
                                    clear_select()
                                    facility.select_flag = 1
                            elif button == 3:
                                facility.select_flag = 0
                    for text in text_list:
                        w = text.w
                        h = text.h
                        x = text.x
                        y = text.y
                        if (x <= xpos <= x+w) and (y <= ypos <= y+h):
                            if button == 1:
                                if text.select_flag == 0:
                                    clear_select()
                                    text.select_flag = 1
                            elif button == 3:
                                text.select_flag = 0
                    for L in lines:
                        (x1,y1) = (L.x1,L.y1)
                        (x2,y2) = (L.x2,L.y2)
                        if x1 == x2 == xpos:
                            if (y1 <= ypos <= y2) or (y2 <= ypos <= y1):
                                if button == 1:
                                    L.select_flag = 1
                                elif button == 3:
                                    L.select_flag = 0    # reset select flag
                        elif y1 == y2 == ypos:
                            if (x1 <= xpos <= x2) or (x2 <= xpos <= x1):
                                if button == 1:
                                    L.select_flag = 1    # set select flag
                                elif button == 3:
                                    L.select_flag = 0    # reset select flag
                        else:
                            mx = round((x1 + x2)/2/grid_unit,0)*grid_unit
                            my = round((y1 + y2)/2/grid_unit,0)*grid_unit
                            if xpos == mx:
                                if ypos == my:
                                    if button == 1:
                                        L.select_flag = 1    # set select flag
                                    elif button == 3:
                                        L.select_flag = 0    # reset select flag
                elif mode == K_l: # draw lines
                    if button == 1: # draw line mode with left mouse button
                        if firstpoint:
                            if firstpoint == (xpos,ypos):
                                firstpoint = []
                            else:
                                lines.append(Line(firstpoint[0],firstpoint[1],xpos,ypos,colorindex,0))
                                firstpoint = (xpos,ypos)
                        else:
                            firstpoint = (xpos,ypos)
                    elif button == 3: # stop line --> back 1 step
                        select_rect = pygame.Rect(0,0,1,1)
                        if firstpoint:
                            firstpoint = []
                        elif lines:
                            lines.pop()
                else:
                    if button == 1: # left mouse button
                        findlist = find_facility(xpos,ypos)
                        if not findlist:
                            if mode == K_p: # point
                                facilities.append(Point(next_No(),xpos,ypos))
                            elif mode == K_o: # point_slow
                                facilities.append(Point_slow(next_No(),xpos,ypos))
                            elif mode == K_s: # sensor
                                facilities.append(Sensor(next_No(),xpos,ypos))
                            elif mode == K_a: # signal3
                                facilities.append(Signal3(next_No(),xpos,ypos))
                            elif mode == K_b: # signal6
                                facilities.append(Signal6(next_No(),xpos,ypos))
                            elif mode == K_t: # tile
                                facilities.append(Tile(0,xpos,ypos,color_index=colorindex))
                            elif mode == K_c: # controller
                                facilities.append(Controller(next_No(),xpos,ypos,color_index=colorindex))
                    elif button == 3: # back 1 step with mouse wheel button
                        select_rect = pygame.Rect(0,0,1,1)
                        if facilities:
                            delete_facility = facilities.pop()
                            delete_No(delete_facility.number)

            # キーイベント処理
            elif event.type == KEYDOWN:
                key = event.key
                select_rect = pygame.Rect(0,0,1,1) # erase rectangle for selection
                if mode == K_l: # draw line mode
                    if key == K_l:
                        colorindex = (colorindex+1)%10 # change line color
                elif mode == K_t: # add terminal mode
                    if key == K_t:
                        colorindex = (colorindex+1)%10 # change controller color
                elif mode == K_c: # add controller mode
                    if key == K_c:
                        colorindex = (colorindex+1)%10 # change controller color

                if key in mode_key_list:
                    mode =key
                if mode != K_m:
                    clear_select()
                if K_0 <= key <= K_9:
                    update_facility_No(key)
                elif key == K_DELETE:
                    delete_lines()
                    delete_facilities()
                elif key == K_LEFT:
                    move_lines(-grid_unit,0)
                    move_text(-grid_unit,0)
                    move_facilities(-grid_unit,0)
                elif key == K_RIGHT:
                    move_lines(grid_unit,0)
                    move_text(grid_unit,0)
                    move_facilities(grid_unit,0)
                elif key == K_UP:
                    move_lines(0,-grid_unit)
                    move_text(0,-grid_unit)
                    move_facilities(0,-grid_unit)
                elif key == K_DOWN:
                    move_lines(0,grid_unit)
                    move_text(0,grid_unit)
                    move_facilities(0,grid_unit)
                elif key == K_PAGEDOWN:
                    window_h = int(window_h/1.4142)
                    window_w = int(window_w/1.4142)
                    BACK_PANEL = pygame.display.set_mode((window_w,window_h))
                    print("window Size = ",window_w,window_h)
                elif key == K_PAGEUP:
                    window_h = int(window_h*1.4142)
                    window_w = int(window_w*1.4142)
                    BACK_PANEL = pygame.display.set_mode((window_w,window_h))
                    print("window Size = ",window_w,window_h)
                elif key == K_f: # flip facility
                    for facility in facilities:
                        if facility.select_flag:
                            facility.flip()
                elif key == K_r: # rotate facility
                    for facility in facilities:
                        if facility.select_flag:
                            facility.rotate()
                elif key == K_w:
                    writefile()
                elif key == K_h:
                    print_help()
                elif key == K_RETURN or key == K_ESCAPE:
                    clear_select()
        if eventflag:
            eventflag = 0
            draw_back()
            draw_firstpoint(firstpoint)
            for L in lines:
                if L.select_flag:
                    w = 15
                else:
                    w = 5
                pygame.draw.line(BACK_PANEL,index2color(L.color_index),(L.x1,L.y1),(L.x2,L.y2),width=w)
            for facility in facilities:
                facility.draw()
            for t in text_list:
                t.draw()
            write_bottom_column(key,colorindex)
            if select_rect.top:
                pygame.draw.rect(BACK_PANEL, (255,0,0),select_rect,width=1)

            pygame.display.update()

if __name__ == '__main__':
    main()
